@@ -1,8 +1,6 @@
 //
 // Extractor.h
 //
-// $Id: //poco/Main/Data/ODBC/include/Poco/Data/ODBC/Extractor.h#5 $
-//
 // Library: Data/ODBC
 // Package: ODBC
 // Module:  Extractor
@@ -20,13 +18,13 @@
 #define Data_ODBC_Extractor_INCLUDED
 
 
-#include "Poco/Data/Constants.h"
 #include "Poco/Data/ODBC/ODBC.h"
-#include "Poco/Data/AbstractExtractor.h"
 #include "Poco/Data/ODBC/Preparator.h"
 #include "Poco/Data/ODBC/ODBCMetaColumn.h"
 #include "Poco/Data/ODBC/Error.h"
 #include "Poco/Data/ODBC/Utility.h"
+#include "Poco/Data/AbstractExtractor.h"
+#include "Poco/Data/Constants.h"
 #include "Poco/Data/Date.h"
 #include "Poco/Data/Time.h"
 #include "Poco/DateTime.h"
@@ -34,6 +32,8 @@
 #include "Poco/Dynamic/Var.h"
 #include "Poco/Nullable.h"
 #include "Poco/UTFString.h"
+#include "Poco/TextEncoding.h"
+#include "Poco/TextConverter.h"
 #include "Poco/Exception.h"
 #include <map>
 #ifdef POCO_OS_FAMILY_WINDOWS
@@ -54,8 +54,9 @@ class ODBC_API Extractor: public Poco::Data::AbstractExtractor
 public:
 	typedef Preparator::Ptr PreparatorPtr;
 
-	Extractor(const StatementHandle& rStmt, 
-		Preparator::Ptr pPreparator);
+	Extractor(const StatementHandle& rStmt,
+		Preparator::Ptr pPreparator,
+		Poco::TextEncoding::Ptr pDBEncoding = nullptr);
 		/// Creates the Extractor.
 
 	~Extractor();
@@ -157,7 +158,7 @@ public:
 	bool extract(std::size_t pos, std::list<Poco::UInt64>& val);
 		/// Extracts an UInt64 list.
 
-#ifndef POCO_LONG_IS_64_BIT
+#ifndef POCO_INT64_IS_LONG
 	bool extract(std::size_t pos, long& val);
 		/// Extracts a long.
 
@@ -306,7 +307,10 @@ public:
 
 	bool extract(std::size_t pos, std::list<Poco::DateTime>& val);
 		/// Extracts a DateTime list.
-	
+
+	bool extract(std::size_t pos, Poco::UUID& val);
+		/// Extracts a UUID.
+
 	bool extract(std::size_t pos, Poco::Any& val);
 		/// Extracts an Any.
 
@@ -341,7 +345,7 @@ public:
 		/// Returns true if the value at [col,row] is null.
 
 	void reset();
-		/// Resets the internally cached length indicators. 
+		/// Resets the internally cached length indicators.
 
 private:
 	static const int CHUNK_SIZE = 1024;
@@ -367,7 +371,7 @@ private:
 	{
 		if (isNull(pos)) return false;
 		poco_assert_dbg (typeid(T) == _pPreparator->at(pos).type());
-		val = *AnyCast<T>(&_pPreparator->at(pos)); 
+		val = *AnyCast<T>(&_pPreparator->at(pos));
 		return true;
 	}
 
@@ -386,7 +390,7 @@ private:
 
 	bool extractBoundImplContainer(std::size_t pos, std::vector<std::string>& values);
 	bool extractBoundImplContainer(std::size_t pos, std::deque<std::string>& values);
-	bool extractBoundImplContainer(std::size_t pos, std::list<std::string>& values); 
+	bool extractBoundImplContainer(std::size_t pos, std::list<std::string>& values);
 	bool extractBoundImplContainer(std::size_t pos, std::vector<Poco::UTF16String>& values);
 	bool extractBoundImplContainer(std::size_t pos, std::deque<Poco::UTF16String>& values);
 	bool extractBoundImplContainer(std::size_t pos, std::list<Poco::UTF16String>& values);
@@ -464,12 +468,12 @@ private:
 	bool extractManualImpl(std::size_t pos, T& val, SQLSMALLINT cType)
 	{
 		SQLRETURN rc = 0;
-		T value = (T) 0;
+		T value{};
 
 		resizeLengths(pos);
 
-		rc = SQLGetData(_rStmt, 
-			(SQLUSMALLINT) pos + 1, 
+		rc = SQLGetData(_rStmt,
+			(SQLUSMALLINT) pos + 1,
 			cType,  //C data type
 			&value, //returned value
 			0,      //buffer length (ignored)
@@ -477,10 +481,10 @@ private:
 
 		if (Utility::isError(rc))
 			throw StatementException(_rStmt, "SQLGetData()");
-		
-		if (isNullLengthIndicator(_lengths[pos])) 
+
+		if (isNullLengthIndicator(_lengths[pos]))
 			return false;
-		else 
+		else
 		{
 			//for fixed-length data, buffer must be large enough
 			//otherwise, driver may write past the end
@@ -569,16 +573,53 @@ private:
 			case MetaColumn::FDT_TIMESTAMP:
 			{ return extAny<T, Poco::DateTime>(pos, val); }
 
-			default: 
+			case MetaColumn::FDT_UUID:
+			{ return extAny<T, Poco::UUID>(pos, val); }
+
+			default:
 				throw DataFormatException("Unsupported data type.");
 		}
 
 		return false;
 	}
 
+	template <typename C>
+	bool stringContainerExtractConvert(std::size_t pos, C& val)
+	{
+		bool ret = false;
+		C res;
+		ret = extractBoundImplContainer(pos, res);
+		val.clear();
+		if (ret)
+		{
+			Poco::TextConverter conv(*_pDBEncoding, *_pToEncoding);
+			val.resize(res.size());
+			typename C::iterator vIt = val.begin();
+			typename C::iterator it = res.begin();
+			for (; it != res.end(); ++it, ++vIt) conv.convert(*it, *vIt);
+		}
+		return ret;
+	}
+
+	template <typename C>
+	bool stringContainerExtract(std::size_t pos, C& val)
+	{
+		bool ret = false;
+		if (Preparator::DE_BOUND == _dataExtraction)
+		{
+			if (!_transcode)
+				ret = extractBoundImplContainer(pos, val);
+			else
+				ret = stringContainerExtractConvert(pos, val);
+		}
+		else
+			throw InvalidAccessException("Direct container extraction only allowed for bound mode.");
+		return ret;
+	}
+
 	bool isNullLengthIndicator(SQLLEN val) const;
-		/// The reason for this utility wrapper are platforms where 
-		/// SQLLEN macro (a.k.a. SQLINTEGER) yields 64-bit value, 
+		/// The reason for this utility wrapper are platforms where
+		/// SQLLEN macro (a.k.a. SQLINTEGER) yields 64-bit value,
 		/// while SQL_NULL_DATA (#define'd as -1 literal) remains 32-bit.
 
 	SQLINTEGER columnSize(std::size_t pos) const;
@@ -587,6 +628,9 @@ private:
 	PreparatorPtr              _pPreparator;
 	Preparator::DataExtraction _dataExtraction;
 	std::vector<SQLLEN>        _lengths;
+	Poco::TextEncoding::Ptr    _pDBEncoding;
+	bool                       _transcode;
+	Poco::TextEncoding::Ptr    _pToEncoding;
 };
 
 
@@ -642,42 +686,42 @@ inline bool Extractor::extractBoundImplContainer(std::size_t pos, std::list<Poco
 }
 
 
-inline bool Extractor::extractBoundImplContainer(std::size_t pos, 
+inline bool Extractor::extractBoundImplContainer(std::size_t pos,
 	std::vector<Poco::Data::CLOB>& values)
 {
 	return extractBoundImplContainerLOB(pos, values);
 }
 
 
-inline bool Extractor::extractBoundImplContainer(std::size_t pos, 
+inline bool Extractor::extractBoundImplContainer(std::size_t pos,
 	std::deque<Poco::Data::CLOB>& values)
 {
 	return extractBoundImplContainerLOB(pos, values);
 }
 
 
-inline bool Extractor::extractBoundImplContainer(std::size_t pos, 
+inline bool Extractor::extractBoundImplContainer(std::size_t pos,
 	std::list<Poco::Data::CLOB>& values)
 {
 	return extractBoundImplContainerLOB(pos, values);
 }
 
 
-inline bool Extractor::extractBoundImplContainer(std::size_t pos, 
+inline bool Extractor::extractBoundImplContainer(std::size_t pos,
 	std::vector<Poco::Data::BLOB>& values)
 {
 	return extractBoundImplContainerLOB(pos, values);
 }
 
 
-inline bool Extractor::extractBoundImplContainer(std::size_t pos, 
+inline bool Extractor::extractBoundImplContainer(std::size_t pos,
 	std::deque<Poco::Data::BLOB>& values)
 {
 	return extractBoundImplContainerLOB(pos, values);
 }
 
 
-inline bool Extractor::extractBoundImplContainer(std::size_t pos, 
+inline bool Extractor::extractBoundImplContainer(std::size_t pos,
 	std::list<Poco::Data::BLOB>& values)
 {
 	return extractBoundImplContainerLOB(pos, values);
@@ -704,7 +748,7 @@ inline void Extractor::reset()
 
 inline void Extractor::resizeLengths(std::size_t pos)
 {
-	if (pos >= _lengths.size()) 
+	if (pos >= _lengths.size())
 		_lengths.resize(pos + 1, (SQLLEN) 0);
 }
 
